import { useEffect, useRef, useState } from 'react';
var DEFAULT_TRESHOLD = 150;
var stubFunction = function () { return undefined; };
export function useInfiniteScroll(_a) {
    var loading = _a.loading, hasNextPage = _a.hasNextPage, _b = _a.isLoaded, isLoaded = _b === void 0 ? function () { return true; } : _b, _c = _a.itemsLength, itemsLength = _c === void 0 ? 0 : _c, onLoadMore = _a.onLoadMore, registerScrollableList = _a.registerScrollableList, _d = _a.threshold, threshold = _d === void 0 ? DEFAULT_TRESHOLD : _d, withPrefetchedData = _a.withPrefetchedData;
    var ref = useRef(null);
    var _e = useState(false), scrollable = _e[0], setScrollable = _e[1];
    var _f = useState(0), scrollbarWidth = _f[0], setScrollWidth = _f[1];
    var _g = useState(0), tableMaxHeight = _g[0], setTableMaxHeight = _g[1];
    var loadMoreItems = loading || !hasNextPage ? stubFunction : onLoadMore;
    var showLoadMore = !loading && !scrollable && hasNextPage;
    useEffect(function () {
        var tbody = ref.current;
        var handleScroll = function () {
            if (isLoaded(itemsLength) &&
                tbody &&
                tbody.scrollTop + tbody.clientHeight >= tbody.scrollHeight - threshold) {
                loadMoreItems();
            }
        };
        if (tbody) {
            setTableMaxHeight(calcMaxTBodyHeight(tbody));
            tbody.addEventListener('scroll', handleScroll);
            return function () {
                tbody.removeEventListener('scroll', handleScroll);
            };
        }
    }, [loadMoreItems, threshold, itemsLength, isLoaded]);
    useEffect(function () {
        // initial fetch
        if (tableMaxHeight === 0 && !withPrefetchedData) {
            loadMoreItems();
        }
    }, [loadMoreItems, tableMaxHeight, withPrefetchedData]);
    useEffect(function () {
        var tbody = ref.current;
        if (!loading && tbody && tableMaxHeight > 0) {
            var tableInnerHeight = calcTBodyInnerHeight(tbody.lastElementChild);
            var scrollableTBody = tableInnerHeight > tableMaxHeight;
            if (scrollableTBody &&
                tableInnerHeight - tableMaxHeight <= getHeight(tbody.lastElementChild)) {
                loadMoreItems();
            }
            if (registerScrollableList) {
                registerScrollableList(ref);
            }
            var width = getScrollbarWidth(tbody);
            setScrollWidth(width);
            setScrollable(scrollableTBody);
        }
    }, [loading, registerScrollableList, tableMaxHeight, loadMoreItems]);
    return {
        ref: ref,
        scrollable: scrollable,
        scrollbarWidth: scrollbarWidth,
        showLoadMore: showLoadMore,
        tableMaxHeight: tableMaxHeight,
    };
}
// Helpers
var getScrollbarWidth = function (tbody) {
    return tbody.clientWidth > 0 ? tbody.offsetWidth - tbody.clientWidth : 0;
};
var getHeight = function (element) {
    return (element === null || element === void 0 ? void 0 : element.clientHeight) || 0;
};
var calcMaxTBodyHeight = function (tbody) {
    var _a;
    var wrapperHeight = getHeight((_a = tbody === null || tbody === void 0 ? void 0 : tbody.parentElement) === null || _a === void 0 ? void 0 : _a.parentElement);
    var headerHeight = getHeight(tbody === null || tbody === void 0 ? void 0 : tbody.previousElementSibling);
    return wrapperHeight - headerHeight;
};
var calcTBodyInnerHeight = function (lastTr) {
    if (lastTr) {
        var lastTrOffset = (lastTr === null || lastTr === void 0 ? void 0 : lastTr.offsetTop) || 0;
        var lastTrHeight = getHeight(lastTr);
        return lastTrOffset + lastTrHeight;
    }
    return 0;
};
